--eMerge 4
--Clinical Data Elements
--Labs-Allergy

--Note: The code below was built in Netezza SQL and may need modification to run on SQL Server, Oracle, etc.

/*
MRN	
String

COUNT_POSITIVE_ALLERGY	
Number of positive unique allergy tests in record	
Integer

ALLERGY_TEST_FLAG	
Flag=1 if [COUNT_POSITIVE_ALLERGY>=2] 
OR [1 COUNT_POSITIVE_ALLERGY and 1 COUNT_POSITIVE_ DISTINCT_ICD_ Allergen_Group] 
OR  [COUNT_POSITIVE_ DISTINCT_ICD_ Allergen_Group >=2] 
; Flag =0 otherwise	
Integer, encoded value
*/

/*
--Will need to determine what values count as positive for the labs--
--Lab codes (OMOP Concept ID)
3007757
3037831
3016459
3006734
3014480
3010354
3023543
3008374
3043730
3006606
3002527
3024149
3028352
3015561
3015627
3012262
3027873
3024593
3023430
3014133
3008136
3040439
3020960
3001915
3036780
3023351
3036780
3024159
3016997
3012711
3016997
3014599
3018352
3015123
3012494
1175134
3009165
3001488
3025260
3002187
3003888
3014126
3005322
3038205
3011951
3005136
3013101
40768481
3013766
3011197
3019658
*/

/*
Allergy to eggs					ICD9 V15.03 or ICD10 Z91.012
Allergy to Peanuts				ICD9 V15.01 or ICD10 Z91.01
Allergy to milk products		ICD10 Z91.011
Food additives allergy status	ICD10 Z91.02
Allergy to seafood				ICD10 Z91.013
Allergy to other foods			ICD10 Z91.018
*/


--Preliminary Step:---------------------------------------------------------------------
--Create a dataset that assigns positive, negative, etc to the values in your lab data.
----------------------------------------------------------------------------------------

select
	M.MEASUREMENT_CONCEPT_ID
	, C.Concept_Name
	, M.VALUE_AS_NUMBER
	, M.RANGE_LOW
	, M.RANGE_HIGH
	, M.MEASUREMENT_SOURCE_VALUE
	, M.VALUE_SOURCE_VALUE
	, M.UNIT_SOURCE_VALUE
	--Values below are from a single site. Change this to specify values pertinent to your site's data.
	--After review, assign appropriate values for "Final_Classification" of 'Positive' result
	, case
		when M.VALUE_SOURCE_VALUE = '<0.10' then 'Negative'
		when M.VALUE_SOURCE_VALUE = ' <0.10' then 'Negative'
		when M.VALUE_SOURCE_VALUE = '<2' and M.Measurement_Source_Value = 'IGE' then 'Negative'
		when M.VALUE_AS_NUMBER <= M.RANGE_HIGH then 'Negative'
		when M.VALUE_AS_NUMBER <= M.RANGE_LOW then 'Negative'
		when M.VALUE_AS_NUMBER > M.RANGE_HIGH then 'Positive'
		when M.VALUE_SOURCE_VALUE = ' >100.00' then 'Positive'
		when M.VALUE_SOURCE_VALUE = '>100.00' then 'Positive'
		when M.VALUE_SOURCE_VALUE IN ('see', 'See', 'QNS', '.', '"', 'SEE', '.>>') then 'Indeterminate'
		when M.VALUE_SOURCE_VALUE IN ('duplicate') then 'Omit'
		else 'TBD'
	  end as Tentative_Classification
	, case
		when M.VALUE_AS_NUMBER > M.RANGE_HIGH then 'Positive'
		when M.VALUE_SOURCE_VALUE = ' >100.00' then 'Positive'
		when M.VALUE_SOURCE_VALUE = '>100.00' then 'Positive'
		else 'Other'
	  end as Final_Classification
	, count(*) as Value_Count
into 
	e4_CDE_Allergy_Lab_Classification_Needed
from 
	V_MEASUREMENT as M
 	left join
	V_Concept as C
		on M.MEASUREMENT_CONCEPT_ID = C.Concept_ID
where M.MEASUREMENT_CONCEPT_ID IN
	(
		3007757, 3037831, 3016459, 3006734, 3014480
		, 3010354, 3023543, 3008374, 3043730, 3006606
		, 3002527, 3024149, 3028352, 3015561, 3015627
		, 3012262, 3027873, 3024593, 3023430, 3014133
		, 3008136, 3040439, 3020960, 3001915, 3036780
		, 3023351, 3036780, 3024159, 3016997, 3012711
		, 3016997, 3014599, 3018352, 3015123, 3012494
		, 1175134, 3009165, 3001488, 3025260, 3002187
		, 3003888, 3014126, 3005322, 3038205, 3011951
		, 3005136, 3013101, 40768481, 3013766, 3011197
		, 3019658
	)
group by
	M.MEASUREMENT_CONCEPT_ID
	, C.Concept_Name
	, M.VALUE_AS_NUMBER
	, M.RANGE_LOW
	, M.RANGE_HIGH
	, M.MEASUREMENT_SOURCE_VALUE
	, M.VALUE_SOURCE_VALUE
	, M.UNIT_SOURCE_VALUE
order by
	M.MEASUREMENT_CONCEPT_ID
	, C.Concept_Name
	, M.VALUE_AS_NUMBER
	, M.RANGE_LOW
	, M.RANGE_HIGH
	, M.MEASUREMENT_SOURCE_VALUE
	, M.VALUE_SOURCE_VALUE
	, M.UNIT_SOURCE_VALUE
;

--select * from e4_CDE_Allergy_Lab_Classification_Needed order by Concept_Name, Value_SOURCE_VALUE

--create final classification table
select
	MEASUREMENT_CONCEPT_ID
	, MEASUREMENT_SOURCE_VALUE
	, Final_Classification
into
	e4_CDE_Allergy_Lab_Classification
from
	e4_CDE_Allergy_Lab_Classification_Needed
;


--------------------------------------
--Lab tests and ICD Categories--
--------------------------------------

--Lab tests
; with cte_allergy_tests as
(
select
	M.PERSON_ID
	, Count(distinct M.MEASUREMENT_CONCEPT_ID) as Count_Positive_Allergy
from 
	v_e4_CDE_Participant_Source_Data as S
	join
	V_MEASUREMENT as M
		on S.Person_ID = M.Person_ID
 	join
	e4_CDE_Allergy_Lab_Classification as C
		on M.MEASUREMENT_CONCEPT_ID = C.MEASUREMENT_CONCEPT_ID
		and C.Final_Classification = 'Positive'
group by
	M.PERSON_ID
)

--ICD Categories
--also with
, cte_ICD_Category_Counts as
(
select
	PERSON_ID 
	, sum(
			case
				when CONDITION_SOURCE_VALUE in ('V15.03', 'Z91.012') then 1 else 0
	    	end
	     )
	as Allergy_Eggs_Count
	, sum(
			case
				when CONDITION_SOURCE_VALUE in ('V15.01', 'Z91.010') then 1 else 0
			end
	     )
	as Allergy_Peanuts_Count
	, sum(
			case
				when CONDITION_SOURCE_VALUE in ('Z91.011') then 1 else 0
			end
	     )
	as Allergy_Milk_Count
	, sum(
			case
				when CONDITION_SOURCE_VALUE in ('Z91.02') then 1 else 0
			end 
		  )
	as Allergy_Food_Additives_Count
	, sum(
			case
				when CONDITION_SOURCE_VALUE in ('Z91.013') then 1 else 0
			end
		 )
	as Allergy_Seafood_Count
	, sum(
			case
				when CONDITION_SOURCE_VALUE in ('Z91.018') then 1 else 0
			end
		 )
	as Allergy_Other_Foods_Count
from
	V_CONDITION_OCCURRENCE as CO
where
	CONDITION_SOURCE_VALUE IN
		(
			'V15.03', 'Z91.012', 'V15.01'
			, 'Z91.010', 'Z91.01', 'Z91.011'
			, 'Z91.02', 'Z91.013', 'Z91.018'
		)
group by
	PERSON_ID
)

--also with
, cte_ICD_Category_Flags as
(
select
	PERSON_ID 
	, case
		when Allergy_Eggs_Count >= 1 then 1 else 0
	  end as Allergy_Eggs_Flag
	, case
		when Allergy_Peanuts_Count >= 1 then 1 else 0
	  end as Allergy_Peanuts_Flag
	, case
		when Allergy_Milk_Count >= 1 then 1 else 0
	  end as Allergy_Milk_Flag
	, case
		when Allergy_Food_Additives_Count >= 1 then 1 else 0
	  end as Allergy_Food_Additives_Flag
	, case
		when Allergy_Seafood_Count >= 1 then 1 else 0
	  end as Allergy_Seafood_Flag
	, case
		when Allergy_Other_Foods_Count >= 1 then 1 else 0
	  end as Allergy_Other_Foods_Flag
from
	cte_ICD_Category_Counts as C
)
		
--also with
, cte_ICD_Categories_Summed as
(
	select
		person_ID
		, Allergy_Eggs_Flag 
		+ Allergy_Peanuts_Flag 
		+ Allergy_Milk_Flag 
		+ Allergy_Food_Additives_Flag 
		+ Allergy_Seafood_Flag 
		+ Allergy_Other_Foods_Flag
		as COUNT_POSITIVE_DISTINCT_ICD_Allergen_Group
	from
		cte_ICD_Category_Flags
)

select
	S.MRN
	, coalesce(A.Count_Positive_Allergy, 0) as Count_Positive_Allergy
	, case
		when A.Count_Positive_Allergy >= 2 then 1
		when ICD.COUNT_POSITIVE_DISTINCT_ICD_Allergen_Group >= 2 then 1
		when A.Count_Positive_Allergy = 1 and ICD.COUNT_POSITIVE_DISTINCT_ICD_Allergen_Group = 1 then 1
		else 0
	  end as Allergy_Test_Flag
into
	e4_CDE_Labs_Allergy
from
	V_e4_CDE_Participant_Source_Data as S
	left join
	cte_allergy_tests as A
		on S.Person_ID = A.Person_ID
	left join
	cte_ICD_Categories_Summed as ICD
		on S.Person_ID = ICD.Person_ID
;



	
	
