--eMerge 4
--Clinical Data Elements
--Labs-Numerical

--Note: The code below was built in Netezza SQL and may need modification to run on SQL Server, Oracle, etc.

/*
MRN

LAB_NAME	
SBP, DBP, total cholesterol, HDL, Triglyceride, or A1C.

DATE_AT_EVENT	
Date event occurred

MEASUREMENT_CONCEPT_ID	
The MEASUREMENT_CONCEPT_ID field is recommended for primary use in analyses, and must be used for network studies.

VALUE_MOST_RECENT	
Pull most recent value in record 

*/


/*
HDL:
3034482
3013473
3053286
3033190
3033638
3011884
3023602
3007070

Total Cholesterol:
3015548
3027114
3031776
3027074
3019900
3049873

Triglyceride:
3007943
3022192
3022038
3030875
3012391
3025839
42868692
3019038
3027997
3048773


A1C:
3004410
3003309
3005673
40762352
3007263
3034639
42869630
40765129

Diastolic BP:
3012888

Systolic BP:
3004249
*/

--------------------------------------
--Lab tests--
--------------------------------------
; with cte_diastolic_bp as
(
	select
		PERSON_ID
		, MRN
		, LAB_NAME
		, MEASUREMENT_CONCEPT_ID
		, DATE_AT_EVENT
		, VALUE_MOST_RECENT
		, UNIT_SOURCE_VALUE
		, Validation
	from
		(
			select
				M.PERSON_ID
				, S.MRN
				, M.MEASUREMENT_CONCEPT_ID
				, C.Concept_Name
				, M.MEASUREMENT_DATE as Date_at_Event
				, M.VALUE_AS_NUMBER as VALUE_MOST_RECENT
				, M.RANGE_LOW
				, M.RANGE_HIGH
				, M.MEASUREMENT_SOURCE_VALUE
				, M.VALUE_SOURCE_VALUE
				, M.UNIT_SOURCE_VALUE
				, 'DBP' as LAB_NAME
				, case
					when M.VALUE_AS_NUMBER < 1 then 'Low'
					else 'Ok'
	  			  end as Validation
				, row_number() over (partition by M.PERSON_ID order by M.MEASUREMENT_DATE desc, M.Measurement_ID desc) as rownum
			from 
				V_e4_CDE_Participant_Source_Data as S
				join
				V_MEASUREMENT as M
					on S.Person_ID = M.Person_ID
 				join
				V_Concept as C
					on M.MEASUREMENT_CONCEPT_ID = C.Concept_ID
			where 
				M.MEASUREMENT_CONCEPT_ID IN
					(
						3012888
					)
		) sub
	where rownum = 1
)

, --also with
cte_systolic_bp as
(
	select
		PERSON_ID
		, MRN
		, LAB_NAME
		, MEASUREMENT_CONCEPT_ID
		, DATE_AT_EVENT
		, VALUE_MOST_RECENT
		, UNIT_SOURCE_VALUE
		, Validation
	from
		(
			select
				M.PERSON_ID
				, S.MRN
				, M.MEASUREMENT_CONCEPT_ID
				, C.Concept_Name
				, M.MEASUREMENT_DATE as Date_at_Event
				, M.VALUE_AS_NUMBER as VALUE_MOST_RECENT
				, M.RANGE_LOW
				, M.RANGE_HIGH
				, M.MEASUREMENT_SOURCE_VALUE
				, M.VALUE_SOURCE_VALUE
				, M.UNIT_SOURCE_VALUE
				, 'SBP' as LAB_NAME
				, case
					when M.VALUE_AS_NUMBER < 60 then 'Low'
					when M.VALUE_AS_NUMBER > 240 then 'High'
					else 'Ok'
	  			  end as Validation
				, row_number() over (partition by M.PERSON_ID order by M.MEASUREMENT_DATE desc) as rownum
			from 
				V_e4_CDE_Participant_Source_Data as S
				join
				V_MEASUREMENT as M
					on S.Person_ID = M.Person_ID
 				join
				V_Concept as C
					on M.MEASUREMENT_CONCEPT_ID = C.Concept_ID
			where 
				M.MEASUREMENT_CONCEPT_ID IN
					(
						3004249
					)
		) sub
	where rownum = 1
)

, --also with
cte_A1C as
(
	select
		PERSON_ID
		, MRN
		, LAB_NAME
		, MEASUREMENT_CONCEPT_ID
		, DATE_AT_EVENT
		, VALUE_MOST_RECENT
		, UNIT_SOURCE_VALUE
		, Validation
	from
		(
			select
				M.PERSON_ID
				, S.MRN
				, M.MEASUREMENT_CONCEPT_ID
				, C.Concept_Name
				, M.MEASUREMENT_DATE as Date_at_Event
				, M.VALUE_AS_NUMBER as VALUE_MOST_RECENT
				, M.RANGE_LOW
				, M.RANGE_HIGH
				, M.MEASUREMENT_SOURCE_VALUE
				, M.VALUE_SOURCE_VALUE
				, M.UNIT_SOURCE_VALUE
				, 'A1C' as LAB_NAME
				, case
					when M.VALUE_AS_NUMBER < 2 then 'Low'
					when M.VALUE_AS_NUMBER > 20 then 'High'
					else 'Ok'
	  			  end as Validation
				, row_number() over (partition by M.PERSON_ID order by M.MEASUREMENT_DATE desc) as rownum
			from 
				V_e4_CDE_Participant_Source_Data as S
				join
				V_MEASUREMENT as M
					on S.Person_ID = M.Person_ID
 				join
				V_Concept as C
					on M.MEASUREMENT_CONCEPT_ID = C.Concept_ID
			where 
				M.MEASUREMENT_CONCEPT_ID IN
					(
						3004410
						, 3003309
						, 3005673
						, 40762352
						, 3007263
						, 3034639
						, 42869630
						, 40765129
					)
		) sub
	where rownum = 1
)

, --also with
cte_HDL as
(
	select
		PERSON_ID
		, MRN
		, LAB_NAME
		, MEASUREMENT_CONCEPT_ID
		, DATE_AT_EVENT
		, VALUE_MOST_RECENT
		, UNIT_SOURCE_VALUE
		, Validation
	from
		(
			select
				M.PERSON_ID
				, S.MRN
				, M.MEASUREMENT_CONCEPT_ID
				, C.Concept_Name
				, M.MEASUREMENT_DATE as Date_at_Event
				, M.VALUE_AS_NUMBER as VALUE_MOST_RECENT
				, M.RANGE_LOW
				, M.RANGE_HIGH
				, M.MEASUREMENT_SOURCE_VALUE
				, M.VALUE_SOURCE_VALUE
				, M.UNIT_SOURCE_VALUE
				, 'HDL' as LAB_NAME
				, case
					when M.VALUE_AS_NUMBER < 5 then 'Low'
					when M.VALUE_AS_NUMBER > 200 then 'High'
					else 'Ok'
	  			  end as Validation
				, row_number() over (partition by M.PERSON_ID order by M.MEASUREMENT_DATE desc) as rownum
			from 
				V_e4_CDE_Participant_Source_Data as S
				join
				V_MEASUREMENT as M
					on S.Person_ID = M.Person_ID
 				join
				V_Concept as C
					on M.MEASUREMENT_CONCEPT_ID = C.Concept_ID
			where 
				M.MEASUREMENT_CONCEPT_ID IN
					(
						3034482
						, 3013473
						, 3053286
						, 3033190
						, 3033638
						, 3011884
						, 3023602
						, 3007070
					)
		) sub
	where rownum = 1
)

, --also with
cte_Triglyceride as
(
	select
		PERSON_ID
		, MRN
		, LAB_NAME
		, MEASUREMENT_CONCEPT_ID
		, DATE_AT_EVENT
		, VALUE_MOST_RECENT
		, UNIT_SOURCE_VALUE
		, Validation
	from
		(
			select
				M.PERSON_ID
				, S.MRN
				, M.MEASUREMENT_CONCEPT_ID
				, C.Concept_Name
				, M.MEASUREMENT_DATE as Date_at_Event
				, M.VALUE_AS_NUMBER as VALUE_MOST_RECENT
				, M.RANGE_LOW
				, M.RANGE_HIGH
				, M.MEASUREMENT_SOURCE_VALUE
				, M.VALUE_SOURCE_VALUE
				, M.UNIT_SOURCE_VALUE
				, 'Triglyceride' as LAB_NAME
				, case
					when M.VALUE_AS_NUMBER < 1 then 'Low'
					when M.VALUE_AS_NUMBER > 500 then 'High'
					else 'Ok'
	  			  end as Validation
				, row_number() over (partition by M.PERSON_ID order by M.MEASUREMENT_DATE desc) as rownum
			from 
				V_e4_CDE_Participant_Source_Data as S
				join
				V_MEASUREMENT as M
					on S.Person_ID = M.Person_ID
 				join
				V_Concept as C
					on M.MEASUREMENT_CONCEPT_ID = C.Concept_ID
			where 
				M.MEASUREMENT_CONCEPT_ID IN
					(
						3007943
						, 3022192
						, 3022038
						, 3030875
						, 3012391
						, 3025839
						, 42868692
						, 3019038
						, 3027997
						, 3048773
					)
		) sub
	where rownum = 1
)

, --also with
cte_Total_Cholesterol as
(
	select
		PERSON_ID
		, MRN
		, LAB_NAME
		, MEASUREMENT_CONCEPT_ID
		, DATE_AT_EVENT
		, VALUE_MOST_RECENT
		, UNIT_SOURCE_VALUE
		, Validation
	from
		(
			select
				M.PERSON_ID
				, S.MRN
				, M.MEASUREMENT_CONCEPT_ID
				, C.Concept_Name
				, M.MEASUREMENT_DATE as Date_at_Event
				, M.VALUE_AS_NUMBER as VALUE_MOST_RECENT
				, M.RANGE_LOW
				, M.RANGE_HIGH
				, M.MEASUREMENT_SOURCE_VALUE
				, M.VALUE_SOURCE_VALUE
				, M.UNIT_SOURCE_VALUE
				, 'Total_Cholesterol' as LAB_NAME
				, case
					when M.VALUE_AS_NUMBER < 50 then 'Low'
					when M.VALUE_AS_NUMBER > 1000 then 'High'
					else 'Ok'
	  			  end as Validation
				, row_number() over (partition by M.PERSON_ID order by M.MEASUREMENT_DATE desc) as rownum
			from 
				V_e4_CDE_Participant_Source_Data as S
				join
				V_MEASUREMENT as M
					on S.Person_ID = M.Person_ID
 				join
				V_Concept as C
					on M.MEASUREMENT_CONCEPT_ID = C.Concept_ID
			where 
				M.MEASUREMENT_CONCEPT_ID IN
					(
						3015548
						, 3027114
						, 3031776
						, 3027074
						, 3019900
						, 3049873
					)
		) sub
	where rownum = 1
)

select * 
into
	e4_CDE_Labs_Numerical
from
(
	select
		S.MRN, 'DBP' as LAB_NAME, MEASUREMENT_CONCEPT_ID, DATE_AT_EVENT, VALUE_MOST_RECENT
	from
		V_e4_CDE_Participant_Source_Data as S
		left join
		cte_diastolic_bp as dbp
			on S.MRN = dbp.MRN
		
	UNION	
		
	select
		S.MRN, 'SBP' as LAB_NAME, MEASUREMENT_CONCEPT_ID, DATE_AT_EVENT, VALUE_MOST_RECENT
	from
		V_e4_CDE_Participant_Source_Data as S		
		left join
		cte_systolic_bp as sbp
			on S.MRN = sbp.MRN
		
	UNION		
		
	select
		S.MRN, 'A1C' as LAB_NAME, MEASUREMENT_CONCEPT_ID, DATE_AT_EVENT, VALUE_MOST_RECENT
	from
		V_e4_CDE_Participant_Source_Data as S	
		left join
		cte_A1C as a1c
			on S.MRN = a1c.MRN
		
	UNION		
		
	select
		S.MRN, 'HDL' as LAB_NAME, MEASUREMENT_CONCEPT_ID, DATE_AT_EVENT, VALUE_MOST_RECENT
	from
		V_e4_CDE_Participant_Source_Data as S	
		left join
		cte_HDL as hdl
			on S.MRN = hdl.MRN
		
	UNION		
		
	select
		S.MRN, 'Triglyceride' as LAB_NAME, MEASUREMENT_CONCEPT_ID, DATE_AT_EVENT, VALUE_MOST_RECENT
	from
		V_e4_CDE_Participant_Source_Data as S	
		left join
		cte_Triglyceride as T
			on S.MRN = T.MRN
		
	UNION		
		
	select
		S.MRN, 'Total_Cholesterol' as LAB_NAME, MEASUREMENT_CONCEPT_ID, DATE_AT_EVENT, VALUE_MOST_RECENT
	from
		V_e4_CDE_Participant_Source_Data as S	
		left join
		cte_Total_Cholesterol as TC
			on S.MRN = TC.MRN
)as U
;


--drop table e4_CDE_Labs_Numerical if exists;




