--eMerge 4
--Clinical Data Elements
--ICD flags for wheezing and eczema

--Note: The code below was built in Netezza SQL and may need modification to run on SQL Server, Oracle, etc.

/*	--Variables
MRN

AGE_AT_FIRST_WHEEZE_EVENT
Age at first occured; Code must be by age 3.0 of the individual 

AGE_AT_SECOND_WHEEZE_EVENT
Age at second occured; Code can be before or after 3 years of the individual 

AGE_AT_FIRST_ECZEMA_EVENT
Age at first occured; Code must be by age 3.0 of the individual 

AGE_AT_SECOND_ECZEMA_EVENT
Age at second occured; Code can be before or after 3 years of the individual 

WHEEZING_FLAG
Flag=1 when age at first wheezing event < or = 3.0 and age at second event is not null; Otherwise flag=0

ECZEMA_FLAG
Flag=1 when age at first eczema event < or = 3.0 and age at second event is not null; Otherwise flag=0
*/

/*
--Eczema--
--ICD9
'691'
'054.0'
'373.31'
'380.22'
'380.23'
'684'
'686.8'
'690.12'
'691.8'
'693.1'
'693.1'
'695.89'
'696.8'
'703.8'
'704.8'
'709.9'
'V13.3'

--ICD10
'B00.0'
'H01.13'
'H60.54'
'H60.6'
'H60.8X'
'L73.8'
'L98.9'
'Z87.2'

'L20.%'
'L21.%'
'L22.%'
'L23.%'
'L24.%'
'L25.%'
'L26.%'
'L27.%'
'L28.%'
'L29.%'
'L30.%'

--Wheezing--
--ICD9
'466.0'
'490'
'491.9'
'519.8'
'786.07'
'995.3'
'786.07'
'V12.69'

--ICD10
'R06.2'
'J20.9'
'J40'
'J42'
'J98.8'
'R06.2'
'Z87.898'
'R21'
'R06.2'
'T78.40XA'
'R06.2'
'T78.X'
*/

--Wheezing--------------------------------------------------

;with temp_first_wheezing
as
(
select
	Person_ID
	, MRN
	, 1 as first_wheezing_event_flag
	, AGE_AT_FIRST_WHEEZE_EVENT
	, Condition_Start_Date
from
(
	select
		S.PERSON_ID
		, S.MRN
		--, extract(year from age(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))) as AGE_AT_FIRST_WHEEZE_EVENT_Years
		, round(months_between(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))/12, 3) as AGE_AT_FIRST_WHEEZE_EVENT
		, CO.CONDITION_OCCURRENCE_ID
		, CO.Condition_Start_Date
		, row_number() over (partition by CO.PERSON_ID order by CO.Condition_Start_Date, CO.Condition_Occurrence_ID) as rownum
	from
		V_CONDITION_OCCURRENCE as CO
		join
		V_e4_CDE_Participant_Source_Data as S
			on CO.PERSON_ID = S.PERSON_ID
	where
		round(months_between(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))/12, 3) <= 3.0
		AND
		(
			CO.CONDITION_SOURCE_VALUE IN 
			(
				'691', '054.0','373.31', '380.22', '380.23', '684', '686.8', '690.12', '691.8'
				, '693.1', '693.1', '695.89', '696.8', '703.8', '704.8', '709.9', 'V13.3'
				,'B00.0', 'H01.13', 'H60.54', 'H60.6', 'H60.8X', 'L73.8', 'L98.9', 'Z87.2'
			)
			OR CO.CONDITION_SOURCE_VALUE like 'L20.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L21.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L22.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L23.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L24.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L25.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L26.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L27.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L28.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L29.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L30.%'
		)
) sub
where
	rownum = 1
)

--also with
, temp_second_wheezing as
(
select
	Person_ID
	, MRN
	, 1 as second_wheezing_event_flag
	, AGE_AT_SECOND_WHEEZE_EVENT
	, Condition_Start_Date
from
(
	select
		S.PERSON_ID
		, S.MRN
		--, extract(year from age(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))) as AGE_AT_Second_WHEEZE_EVENT_Years
		, round(months_between(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))/12, 3) as AGE_AT_SECOND_WHEEZE_EVENT
		, CO.CONDITION_OCCURRENCE_ID
		, CO.Condition_Start_Date
		, row_number() over (partition by CO.PERSON_ID order by CO.Condition_Start_Date, CO.Condition_Occurrence_ID) as rownum
	from
		V_CONDITION_OCCURRENCE as CO
		join
		V_e4_CDE_Participant_Source_Data as S
			on CO.PERSON_ID = S.PERSON_ID
		join
		temp_first_wheezing as W1
			on CO.Person_ID = W1.Person_ID
			and CO.Condition_Start_Date > W1.Condition_Start_Date
	where
		(
			CO.CONDITION_SOURCE_VALUE IN 
			(
				'691', '054.0','373.31', '380.22', '380.23', '684', '686.8', '690.12', '691.8'
				, '693.1', '693.1', '695.89', '696.8', '703.8', '704.8', '709.9', 'V13.3'
				,'B00.0', 'H01.13', 'H60.54', 'H60.6', 'H60.8X', 'L73.8', 'L98.9', 'Z87.2'
			)
			OR CO.CONDITION_SOURCE_VALUE like 'L20.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L21.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L22.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L23.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L24.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L25.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L26.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L27.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L28.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L29.%'
			OR CO.CONDITION_SOURCE_VALUE like 'L30.%'
		)
) sub
where
	rownum = 1
)




--also with
, temp_wheezing as
(
select
	S.MRN
	, W1.AGE_AT_FIRST_WHEEZE_EVENT
	, W2.AGE_AT_SECOND_WHEEZE_EVENT
	, Case
		when 
			W1.AGE_AT_FIRST_WHEEZE_EVENT is not null
			and W2.AGE_AT_SECOND_WHEEZE_EVENT is not null
			then 1 
			else 0
	  end as WHEEZING_FLAG
from
	V_e4_CDE_Participant_Source_Data as S
	left join
	temp_first_wheezing as W1
		on S.MRN = W1.MRN
	left join
	temp_second_wheezing as W2
		on S.MRN = W2.MRN
)

--Eczema--------------------------------------------------------------

--also with
, temp_first_Eczema as
(
select
	Person_ID
	, MRN
	, 1 as first_Eczema_event_flag
	, AGE_AT_FIRST_Eczema_EVENT
	, Condition_Start_Date
from
(
	select
		S.PERSON_ID
		, S.MRN
		--, extract(year from age(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))) as AGE_AT_FIRST_Eczema_EVENT_Years
		, round(months_between(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))/12, 3) as AGE_AT_First_Eczema_EVENT
		, CO.CONDITION_OCCURRENCE_ID
		, CO.Condition_Start_Date
		, row_number() over (partition by CO.PERSON_ID order by CO.Condition_Start_Date, CO.Condition_Occurrence_ID) as rownum
	from
		V_CONDITION_OCCURRENCE as CO
		join
		V_e4_CDE_Participant_Source_Data as S
			on CO.PERSON_ID = S.PERSON_ID
	where
		round(months_between(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))/12, 3) <= 3.0
		AND
		CO.CONDITION_SOURCE_VALUE IN 
			(
				'466.0', '490', '491.9', '519.8', '786.07', '995.3', '786.07', 'V12.69'
				, 'R06.2', 'J20.9', 'J40', 'J42', 'J98.8', 'R06.2', 'Z87.898', 'R21', 'R06.2', 'T78.40XA', 'R06.2', 'T78.X'
			)
) sub
where
	rownum = 1
)

--also with
, temp_second_Eczema as
(
select
	Person_ID
	, MRN
	, 1 as second_Eczema_event_flag
	, AGE_AT_SECOND_Eczema_EVENT
	, Condition_Start_Date
from
(
	select
		S.PERSON_ID
		, S.MRN
		--, extract(year from age(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))) as AGE_AT_SECOND_Eczema_EVENT_Years
		, round(months_between(to_date(CO.Condition_Start_Date,'YYYY-MM-DD'),to_date(S.ehr_Date_of_Birth,'YYYY-MM-DD'))/12, 3) as AGE_AT_SECOND_Eczema_EVENT
		, CO.CONDITION_OCCURRENCE_ID
		, CO.Condition_Start_Date
		, row_number() over (partition by CO.PERSON_ID order by CO.Condition_Start_Date, CO.Condition_Occurrence_ID) as rownum
	from
		V_CONDITION_OCCURRENCE as CO
		join
		V_e4_CDE_Participant_Source_Data as S
			on CO.PERSON_ID = S.PERSON_ID
		join
		temp_first_eczema as E1
			on CO.Person_ID = E1.Person_ID
			and CO.Condition_Start_Date > E1.Condition_Start_Date
	where
		CO.CONDITION_SOURCE_VALUE IN 
			(
				'466.0', '490', '491.9', '519.8', '786.07', '995.3', '786.07', 'V12.69'
				, 'R06.2', 'J20.9', 'J40', 'J42', 'J98.8', 'R06.2', 'Z87.898', 'R21', 'R06.2', 'T78.40XA', 'R06.2', 'T78.X'
			)
) sub
where
	rownum = 1
)

--also with
, temp_Eczema as
(
select
	S.MRN
	, E1.AGE_AT_FIRST_Eczema_EVENT
	, E2.AGE_AT_SECOND_Eczema_EVENT
	, Case
		when 
			E1.AGE_AT_FIRST_Eczema_EVENT is not null
			and E2.AGE_AT_SECOND_Eczema_EVENT is not null
			then 1 
			else 0
	  end as Eczema_FLAG
from
	V_e4_CDE_Participant_Source_Data as S
	left join
	temp_first_Eczema as E1
		on S.MRN = E1.MRN
	left join
	temp_second_Eczema as E2
		on S.MRN = E2.MRN
)


--Combine--------------------------------------

select
	S.MRN
	, W.AGE_AT_FIRST_Wheeze_EVENT
	, W.AGE_AT_SECOND_Wheeze_EVENT
	, E.AGE_AT_FIRST_Eczema_EVENT
	, E.AGE_AT_SECOND_Eczema_EVENT
	, W.Wheezing_Flag
	, E.Eczema_Flag
into 
	e4_CDE_ICD_Flags
from
	V_e4_CDE_Participant_Source_Data as S
	left join
	temp_Wheezing as W
		on S.MRN = W.MRN
	left join
	temp_Eczema as E
		on S.MRN = E.MRN
;

--select * from e4_CDE_ICD_Flags;
--drop table e4_CDE_ICD_Flags if exists;



