--eMerge 4
--Clinical Data Elements
--Combine variables from the other scripts

--Note: The code below was built in Netezza SQL and may need modification to run on SQL Server, Oracle, etc.

--drop table E4_CDE_Combined if exists;

select
	S.mrn
	,S.ehr_participant_first_name
	,S.ehr_participant_last_name
	,S.ehr_date_of_birth
	,NULL as heart_attack_coronary_artery_disease
	,NULL as bmi_adult
	,L2.count_positive_allergy
	,L2.ALLERGY_TEST_FLAG
	
	,max('HDL') as hdl_lab_name
	,max(case
			when LAB_NAME = 'HDL' then DATE_AT_EVENT else '1900-01-01'
 	 	 end
		) as hdl_date_at_event 
	,min(case
			when LAB_NAME = 'HDL' then MEASUREMENT_CONCEPT_ID else 99999999
 	 	 end
		) as hdl_measurement_concept_id
	,round(max(case
			when LAB_NAME = 'HDL' then VALUE_MOST_RECENT else -9
	 	 end)) as hdl_value_most_recent
	 
	,max('Total_Cholesterol') as totalcholest_lab_name
	,max(case
			when LAB_NAME = 'Total_Cholesterol' then DATE_AT_EVENT else '1900-01-01'
 	 	 end
		) as totalcholest_date_at_event 
	,min(case
			when LAB_NAME = 'Total_Cholesterol' then MEASUREMENT_CONCEPT_ID else 99999999
 	 	 end
		) as totalcholest_measurement_concept_id
	,round(max(case
			when LAB_NAME = 'Total_Cholesterol' then VALUE_MOST_RECENT else -9
 	 	 end
		)) as totalcholest_value_most_recent
	 
	,max('Triglyceride') as triglyceride_lab_name
	,max(case
			when LAB_NAME = 'Triglyceride' then DATE_AT_EVENT else '1900-01-01'
 	 	 end
		) as triglyceride_date_at_event 
	,min(case
			when LAB_NAME = 'Triglyceride' then MEASUREMENT_CONCEPT_ID else 99999999
 	 	 end
		) as triglyceride_measurement_concept_id
	,round(max(case
			when LAB_NAME = 'Triglyceride' then VALUE_MOST_RECENT else -9
 	 	 end
		)) as triglyceride_value_most_recent
	 
	,max('A1C') as a1c_lab_name
	,max(case
			when LAB_NAME = 'A1C' then DATE_AT_EVENT else '1900-01-01'
 	 	 end
		) as a1c_date_at_event 
	,min(case
			when LAB_NAME = 'A1C' then MEASUREMENT_CONCEPT_ID else 99999999
 	 	 end
		) as a1c_measurement_concept_id
	,round(max(case
			when LAB_NAME = 'A1C' then VALUE_MOST_RECENT else -9
 	 	 end
		),1) as a1c_value_most_recent
	 
	,max('LDL') as ldl_lab_name
	,max('1900-01-01')as ldl_date_at_event
	,max(99999999)as ldl_measurement_concept_id
	,max(-9) as ldl_value_most_recent
	
	,max('DBP') as dbp_lab_name
	,max(case
			when LAB_NAME = 'DBP' then DATE_AT_EVENT else '1900-01-01'
 	 	 end
		) as dbp_date_at_event 
	,min(case
			when LAB_NAME = 'DBP' then MEASUREMENT_CONCEPT_ID else 99999999
 	 	 end
		) as dbp_measurement_concept_id
	,round(max(case
			when LAB_NAME = 'DBP' then VALUE_MOST_RECENT else -9
 	 	 end
		)) as dbp_value_most_recent
	 
	,max('SBP') as sbp_lab_name
	,max(case
			when LAB_NAME = 'SBP' then DATE_AT_EVENT else '1900-01-01'
 	 	 end
		) as sbp_date_at_event 
	,min(case
			when LAB_NAME = 'SBP' then MEASUREMENT_CONCEPT_ID else 99999999
 	 	 end
		) as sbp_measurement_concept_id
	,round(max(case
			when LAB_NAME = 'SBP' then VALUE_MOST_RECENT else -9
 	 	 end
		)) as sbp_value_most_recent
	 
	,coalesce(I.age_at_first_wheeze_event, -9) as age_at_first_wheeze_event
	,coalesce(I.age_at_second_wheeze_event, -9) as age_at_second_wheeze_event
	,I.wheezing_flag
	,coalesce(I.age_at_first_eczema_event, -9) as age_at_first_eczema_event
	,coalesce(I.age_at_second_eczema_event, -9) as age_at_second_eczema_event
	,I.eczema_flag 
into
	E4_CDE_Combined
from
	V_e4_CDE_Participant_Source_Data as S
	left join
	E4_CDE_LABS_NUMERICAL as L1
		on S.mrn = L1.mrn
	left join
	E4_CDE_LABS_ALLERGY as L2
		on S.mrn = L2.mrn
	left join
	E4_CDE_ICD_FLAGS as I
		on S.mrn = I.mrn
group by
	S.mrn
	,S.ehr_participant_first_name
	,S.ehr_participant_last_name
	,S.ehr_date_of_birth
	,L2.count_positive_allergy
	,L2.ALLERGY_TEST_FLAG
	,I.age_at_first_wheeze_event
	,I.age_at_second_wheeze_event
	,I.wheezing_flag
	,I.age_at_first_eczema_event
	,I.age_at_second_eczema_event
	,I.eczema_flag 
order by S.MRN
;


/*
--export to csv for test of import into REDCap
CREATE EXTERNAL TABLE 'C:\\Users\\dornca\\OneDrive - VUMC\\Documents\\A1_Projects\\eMerge\\eMerge4_Output\\e4_CDE_Combined_Test_Import.csv'
USING
(
	delimiter ','
	Y2BASE 2000
	ENCODING 'internal'
	REMOTESOURCE 'ODBC'
	ESCAPECHAR '\'
	INCLUDEHEADER
)
AS 

SELECT *
  FROM E4_CDE_Combined
;